﻿drop sequence if exists seq_transfer_node_id;
create sequence seq_transfer_node_id start with 1;
select setval('seq_transfer_node_id', (select max(id)+1 from tempus.road_node));

drop sequence if exists seq_transfer_section_id;
create sequence seq_transfer_section_id start with 1;
select setval('seq_transfer_section_id', (select max(id)+1 from tempus.road_section));


-- mark each (distinct) pt_stop involved in a transfer
drop table if exists _tempus_import.pt_transfer_new_nodes;
create table _tempus_import.pt_transfer_new_nodes as

with transfer_stops as
(
select
  distinct pt_stop.id as stop_id
from
  tempus.pt_stop
  , _tempus_import.transfers
  join _tempus_import.pt_stop_idmap idmap1 on idmap1.vendor_id = from_stop_id
  join _tempus_import.pt_stop_idmap idmap2 on idmap2.vendor_id = to_stop_id
where
  transfer_type::integer = 2
  and
  (
    pt_stop.id = idmap1.id
    or
    pt_stop.id = idmap2.id
  )
)
, split_nodes as
(
select
  stop_id
  , rs.id as road_section_id
  , st_lineinterpolatepoint(rs.geom, st_linelocatepoint(rs.geom, pt_stop.geom)) as geom
from
  transfer_stops
  join tempus.pt_stop on pt_stop.id = stop_id
  join tempus.road_section rs on rs.id = pt_stop.road_section_id
)
select
  stop_id
  -- a new road_node_id that split the existing road_section
  , nextval('seq_transfer_node_id')::bigint as split_node_id
  -- the new splitted road_section
  , nextval('seq_transfer_section_id')::bigint as new_rs_id 
  -- the current road_section
  , rs.id as current_rs_id
  , rs.node_from as node_from
  , rs.node_to as node_to
  , split_nodes.geom as split_node_geom
  , pt_stop.abscissa_road_section as abscissa
  , st_linesubstring(rs.geom, 0.0, pt_stop.abscissa_road_section) as first_split
  , st_linesubstring(rs.geom, pt_stop.abscissa_road_section, 1.0) as second_split
from
  split_nodes
  join tempus.road_section rs on rs.id = road_section_id
  join tempus.pt_stop on pt_stop.id = stop_id
;


-- insert a new node in the middle of the attached road_section
insert into tempus.road_node (id, bifurcation, geom)
select
  split_node_id
  , false as bifurcation
  , split_node_geom
from
  _tempus_import.pt_transfer_new_nodes
where
  geometrytype(first_split) <> 'POINT'
  and geometrytype(second_split) <> 'POINT'
;

-- update the first part of the splitted road_section
update
  tempus.road_section
set
  geom = nn.first_split
  , node_to = nn.node_to
  , length = road_section.length * nn.abscissa
from
  _tempus_import.pt_transfer_new_nodes nn
where
  road_section.id = current_rs_id
  and geometrytype(first_split) <> 'POINT';
  and geometrytype(second_split) <> 'POINT'
;

-- insert the second part of the split section
insert into
	tempus.road_section
        (id, road_type, node_from, node_to, traffic_rules_ft, traffic_rules_tf, length, car_speed_limit, road_name, lane, roundabout, bridge, tunnel, ramp, tollway, geom)
select
   new_rs_id,
   rs.road_type,
   split_node_id as node_from,
   rs.node_to,
   rs.traffic_rules_ft,
   rs.traffic_rules_tf,
   rs.length * (1 - abscissa),
   rs.car_speed_limit,
   rs.road_name,
   rs.lane,
   rs.roundabout,
   rs.bridge,
   rs.tunnel,
   rs.ramp,
   rs.tollway,
   second_split
from
    _tempus_import.pt_transfer_new_nodes nn
    join tempus.road_section rs on rs.id = nn.current_rs_id
where
    geometrytype(first_split) <> 'POINT'
    and geometrytype(second_split) <> 'POINT'
;

-- for road_sections that cannot be split (pt stop too close to one extremity)
-- select the right node
update _tempus_import.pt_transfer_new_nodes
set
  split_node_id = rs.node_from
  , split_node_geom = st_startpoint(rs.geom)
from
  tempus.road_section rs
where
  rs.id = current_rs_id
  and geometrytype(first_split) = 'POINT';

update _tempus_import.pt_transfer_new_nodes
set
  split_node_id = rs.node_to
  , split_node_geom = st_endpoint(rs.geom)
from
  tempus.road_section rs
where
  rs.id = current_rs_id
  and geometrytype(second_split) = 'POINT';
  
insert into
	tempus.road_section
        (id, road_type, node_from, node_to, traffic_rules_ft, traffic_rules_tf, length, car_speed_limit, road_name, lane, roundabout, bridge, tunnel, ramp, tollway, geom)
select
  nextval('seq_transfer_section_id')::bigint
  , 5 as road_type -- "other"
  , nn1.split_node_id
  , nn2.split_node_id
  , 1
  , 0
  , min_transfer_time::float * 5000 / 3600.0 as length -- convert from time to distance (multiply by walking speed)
  , 0
  , 'Transfer'
  , 1
  , false
  , false
  , false
  , false
  , false
  , st_makeline(nn1.split_node_geom, nn2.split_node_geom)
from
  _tempus_import.transfers
  join _tempus_import.pt_stop_idmap idmap1 on idmap1.vendor_id = from_stop_id
  join _tempus_import.pt_stop_idmap idmap2 on idmap2.vendor_id = to_stop_id
  join _tempus_import.pt_transfer_new_nodes nn1 on nn1.stop_id = idmap1.id
  join _tempus_import.pt_transfer_new_nodes nn2 on nn2.stop_id = idmap2.id
;  